"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/middleware.index.ts
var middleware_index_exports = {};
__export(middleware_index_exports, {
  cacheCalls: () => cacheCalls,
  includeAssistantMessage: () => includeAssistantMessage
});
module.exports = __toCommonJS(middleware_index_exports);

// src/model.middleware.ts
var import_ai = require("ai");

// src/message.builder.ts
var import_resolved = require("@synstack/resolved");
var import_ts_pattern = require("ts-pattern");
var import_text = require("@synstack/text");
var import_fs = require("fs");
function mapTextPart(v) {
  return { type: "text", text: v };
}
function mapImagePart(v) {
  return {
    type: "image",
    mimeType: v.mimeType,
    image: v.encoding === "path" ? (0, import_fs.readFileSync)(v.data) : v.data
  };
}
function mapFilePart(v) {
  return {
    type: "file",
    mimeType: v.mimeType,
    data: v.encoding === "path" ? (0, import_fs.readFileSync)(v.data) : v.data
  };
}
function userMsgWithOptions(options) {
  return function userMsg2(template, ...values) {
    return (0, import_resolved.pipe)((0, import_text.t)(template, ...values))._(import_text.tParse)._(
      (parts) => parts.map(
        (v) => (0, import_ts_pattern.match)(v).with(import_ts_pattern.P.string, mapTextPart).with({ type: "image" }, mapImagePart).with({ type: "file" }, mapFilePart).exhaustive()
      )
    )._(
      (content) => ({
        role: "user",
        content,
        providerOptions: options.providerOptions
      })
    ).$;
  };
}
function userMsg(template, ...values) {
  return userMsgWithOptions({})(template, ...values);
}
userMsg.cached = userMsgWithOptions({
  providerOptions: {
    anthropic: { cacheControl: { type: "ephemeral" } },
    bedrock: { cachePoint: { type: "default" } }
  }
});
var messageToText = (message) => {
  if (typeof message.content === "string") return message.content;
  return message.content.reduce((v, c) => {
    if (c.type === "text") return v + c.text;
    return v;
  }, "");
};

// src/model.middleware.ts
var includeAssistantMessage = (model) => (0, import_ai.wrapLanguageModel)({
  model,
  middleware: {
    wrapGenerate: async ({ doGenerate, params }) => {
      const lastMessage = params.prompt.at(-1);
      const result = await doGenerate();
      if (lastMessage?.role === "assistant") {
        return {
          ...result,
          text: messageToText(lastMessage) + (result.text ?? "")
        };
      }
      return result;
    },
    wrapStream: async ({ doStream, params }) => {
      const lastMessage = params.prompt.at(-1);
      const result = await doStream();
      if (lastMessage?.role !== "assistant") {
        return result;
      }
      const outputStream = result.stream.pipeThrough(
        new TransformStream({
          start(controller) {
            controller.enqueue({
              type: "text-delta",
              textDelta: messageToText(lastMessage)
            });
          }
        })
      );
      return {
        rawCall: result.rawCall,
        rawResponse: result.rawResponse,
        request: result.request,
        warnings: result.warnings,
        stream: outputStream
      };
    }
  }
});
var cacheCalls = (cache) => (model) => {
  return (0, import_ai.wrapLanguageModel)({
    model,
    middleware: {
      wrapGenerate: async ({ doGenerate, params }) => {
        const [hit, value] = await cache.get([params]);
        if (hit === "hit") {
          return value;
        }
        const res = await doGenerate();
        await cache.set([params], res);
        return res;
      },
      wrapStream: async ({ doStream, params }) => {
        const [hit, value] = await cache.get([params]);
        if (hit === "hit") {
          let index = 0;
          return {
            ...value,
            stream: new ReadableStream({
              pull(controller) {
                if (index < value.stream.length) {
                  controller.enqueue(value.stream[index]);
                  index++;
                } else {
                  controller.close();
                }
              }
            })
          };
        }
        const res = await doStream();
        const collector = [];
        res.stream.pipeThrough(
          new TransformStream({
            transform(chunk, controller) {
              collector.push(chunk);
              controller.enqueue(chunk);
            }
          })
        );
        await cache.set([params], {
          ...res,
          stream: collector
        });
        return res;
      }
    }
  });
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  cacheCalls,
  includeAssistantMessage
});
//# sourceMappingURL=middleware.index.cjs.map